//
//  BMediaKit.h
//  kcrtu
//
//  Created by mark on 17/12/12.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import <UIKit/UIKit.h>

typedef NS_ENUM(NSUInteger, MapleRtcRenderMode) {
    // 如果视频尺寸与显示视窗尺寸不一致，则视频流会按照显示视窗的比例进行周边裁剪填满视窗。
    RENDER_MODE_HIDDEN   = 1,
    // 如果视频尺寸与显示视窗尺寸不一致，在保持长宽比的前提下，将视频进行缩放后填满视窗。
    RENDER_MODE_FIT      = 2
};

typedef NS_ENUM(NSInteger, MapleRtcErrorCode) {
    KcMedia_Error_Enter_Unknown      = 9000,
};

typedef NS_ENUM(NSInteger, MapleLivePublishStatus) {
    LIVE_PUBLISH_DISCONNECTED = 0,
    LIVE_PUBLISH_CONNECTING = 1,
    LIVE_PUBLISH_CONNECTED = 2,
};

typedef NS_ENUM(NSInteger, MapleLivePlayStatus) {
    LIVE_PLAY_DISCONNECTED = 0,
    LIVE_PLAY_CONNECTING = 1,
    LIVE_PLAY_CONNECTED = 2,
};


typedef NS_ENUM(NSInteger, MapleRtcChannelProfile) {
     CHANNEL_PROFILE_VOICE_ONLY = 0,
     CHANNEL_PROFILE_VOICE_VIDEO = 1,
     CHANNEL_PROFILE_LIVE_VIDEO = 2,
     CHANNEL_PROFILE_LIVE_LOOK = 3
};

typedef NS_ENUM(NSInteger, MapleRtcVideoConferenceProfile) {	 
	VIDEO_CONFERENCE_PROFILE_NORMAL = 1,
    VIDEO_CONFERENCE_PROFILE_SFU = 2,
    VIDEO_CONFERENCE_PROFILE_MCU = 3,
};

typedef NS_ENUM(NSInteger, MapleRtcAudioConferenceProfile) {
    AUDIO_CONFERENCE_PROFILE_NORMAL = 1,
    AUDIO_CONFERENCE_PROFILE_SFU = 2,
    AUDIO_CONFERENCE_PROFILE_MCU = 3,
};


typedef NS_ENUM(NSInteger, MapleRtcVideoProfile) {
                                               // res       fps  kbps
    VIDEO_PROFILE_Invalid = -1,
    VIDEO_PROFILE_240P = 20,            // 320x240   20   300
    VIDEO_PROFILE_360P = 30,            // 640x360   20   400
    VIDEO_PROFILE_480P = 40,            // 640x480   20   500
    VIDEO_PROFILE_DEFAULT = VIDEO_PROFILE_360P,
};

typedef NS_ENUM(NSInteger, MapleRtcAudioProfile) {
    AUDIO_PROFILE_DEFAULT = 1,                // use default settings
    AUDIO_PROFILE_VOICE_STANDARD = 1,
    AUDIO_PROFILE_VOICE_HIGH_QUALITY = 2,
    AUDIO_PROFILE_MUSIC_STANDARD = 3,
    AUDIO_PROFILE_MUSIC_HIGH_QUALITY = 4,
};

typedef NS_ENUM(NSInteger, MapleRtcWarningCode) {
    KcMedia_Warn_Default = 8000
};

typedef NS_ENUM(NSInteger, MapleRtcJoinChannelFailedReason) {
    KC_JoinChannelFailed = 4000,
    KC_LoginSignalServerFailed = 4001,
    KC_ConnectMediaServerFailed = 4003,
    KC_AppIdInvalid = 4005,
    KC_AppIdDisabled = 4006
    
};

typedef NS_ENUM(NSInteger, MapleRtcLeaveChannelReason) {
    KC_LeaveChannelFailed = 4100
};

typedef NS_ENUM(NSInteger, MapleRtcKickedOutReason) {
    KC_KickedOutReLogin = 4200,
    KC_KickedOutTimeOut = 4201
};

typedef NS_ENUM(NSInteger, MapleRtcVideoSourceType) {
    VIDEO_SOURCE_NULL = -1,
    //VIDEO_SOURCE_FRONT_CAMERA = 1,
    //VIDEO_SOURCE_BACK_CAMERA = 2,
    VIDEO_SOURCE_CAMERA = 1,
    VIDEO_SOURCE_MEDIA = 2,
};

__attribute__((visibility("default"))) @interface BMVideoCanvas : NSObject
@property (strong, nonatomic) UIView* view;           // 视频显示窗口。SDK不维护view的生命周期，应用程序应保证view在通话中是有效的。
@property (assign, nonatomic) MapleRtcRenderMode renderMode; // 视频显示模式
@property (assign, nonatomic) NSString* uid;              // 用户ID

- (instancetype)init:(UIView*)view withMode:(MapleRtcRenderMode)mode withUid:(NSString*) uid;
@end




@class BMediaKit;

/**
 *  视频发布状态回调
 */
@protocol BMediaDelegate <NSObject>
@optional


/**
 *  加入频道成功回调
 *
 *  @param channel 频道名
 *  @param uid     用户ID
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onJoinChannelSuccess:(NSString* _Nonnull)channelId withUid:(NSString* _Nonnull)uid;

/**
 *  加入频道失败回调
 *
 *  @param reason 失败原因
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onJoinChannelFailed:(MapleRtcJoinChannelFailedReason)reason;


/**
 *  发生警告回调
 *  通常情况下，SDK上报的错误意味着SDK能自动恢复，不需要应用程序干预或提示用户。
 *
 *  @param warningCode 错误代码
 */

- (void)rtcEngine:(BMediaKit * _Nonnull)engine onWarning:(MapleRtcWarningCode)warningCode;

/**
 *  发生错误回调
 *  通常情况下，SDK上报的错误意味着SDK无法自动恢复，需要应用程序干预或提示用户。
 *
 *  @param errorCode 错误代码
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onError:(MapleRtcErrorCode)errorCode;


/**
 *  成功离开频道的回调
 *
 *  @param reason 离开原因
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onLeaveChannel:(MapleRtcLeaveChannelReason)reason;


/**
 *  用户加入回调
 *
 *  @param uids        所有加入用户ID列表
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onUserJoinedNotice:(NSArray * _Nonnull)uids;

/**
 *  用户离线回调
 *
 *  @param uids    所有离开用户ID列表
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onUserOfflineNotice:(NSArray * _Nonnull)uids;

/**
 *  用户被踢出回调
 *
 *  @param reason 当前用户被踢出的原因
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onForceKickOutChannel:(MapleRtcKickedOutReason)reason;

/**
 *  提示当前用户说话音量
 *
 *  @param engine      The engine kit
 *  @param volume      所有人说话音量总和(0~255)
 *  @param volumeInfo  说话者音量信息(0~255)
 *  注：
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onAudioVolumeIndication:(NSDictionary* _Nonnull)volumeInfo totalVolume:(NSInteger)volume;


/**
 *  远端首帧视频流解码尺寸改变回调
 *
 *  @param uid     用户ID，指定是哪个用户的视频流
 *  @param size    视频流尺寸（宽度和高度）
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onFirstRemoteVideoFrameSizeChanged:(NSString* _Nonnull)uid size:(CGSize)size;


/**
 *  网络连接丢失回调
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onConnectionLost:(MapleRtcWarningCode)warningCode;



- (void)rtcEngine:(BMediaKit * _Nonnull)engine onMediaStop:(int)index withPath:(NSString* _Nonnull)path;

/*
 推流cdn 状态
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onLivePublishStatus:(MapleLivePublishStatus)status;

/*
 播放网络流 状态
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onLivePlayStatus:(MapleLivePlayStatus)status;

/*
 
 */
- (void)rtcEngine:(BMediaKit * _Nonnull)engine onLocalVideoSourceChanged:(MapleRtcVideoSourceType)type withStatus:(int)status;


@end


__attribute__((visibility("default"))) @interface BMediaKit : NSObject

@property (nonatomic, weak)   id <BMediaDelegate> _Nullable  delegate;
/**
 *  初始化SDK，加载资源。
 *
 *  @param appId    应用ID，由平台分配，用于区分不同的客户和应用，在同一个平台内保证唯一。
 *  @param delegate SDK回调代理
 *
 *  @return TTTRtc 对象(单例)
 */
+ (instancetype)sharedEngineWithAppId:(NSString * _Nonnull)appId
                             delegate:(id<BMediaDelegate> _Nullable)delegate;
							 
/**
 *  设置参数
 *
 *  @param key 参数key
 *
 *  @param value 参数value
 *
 */							 

- (void) setParameter:(NSString * _Nonnull)key withValue:(NSString * _Nonnull)value;

/**
 *  设置频道模式
 *
 *  @param profile 频道模式
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int) setChannelProfile:(NSInteger)profile;

- (int) setVideoLayout:(NSInteger)value;
/**
 *  设置视频会议属性
 *
 *  @param profile 视频会议属性
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
 
- (int) setVideoConferenceProfile:(NSInteger)profile;


/*
  设置服务器分发类型
 */
- (int) setAudioConferenceProfile:(NSInteger)audioprofile;

/**
 *  设置媒体属性
 *
 *  @param audioprofile 音频属性
 *
 *  @param videoprofile 视频属性
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */

- (int) setMediaProfile:(NSInteger)audioProfile withVideo:(NSInteger)videoProfile;

/**
 *  设置加入频道用户Id
 *
 *  @param uid 用户ID
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */

- (int) setJoinChannelUserId:(NSString * _Nonnull)uid;

/**
 *  获取SDK版本号
 */
+ (NSUInteger)getSdkVersion;


- (void)enableDebugLogging:(BOOL)enabled;

/**
 *  加入通话频道
 *
 *  @param channelId       标识通话的频道名称，长度在32字节以内的字符串。
 *  @param uid               用户ID
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)joinChannel:(NSString * _Nonnull)channelId
                    uid:(NSString * _Nonnull)uid;

/**
 *  加入通话频道
 *
 *  @param channelId       标识通话的频道名称，长度在32字节以内的字符串。
 *  @param uid               用户ID
 *  @param mic               加入房间时是否打开麦克风
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)joinChannel:(NSString * _Nonnull)channelId
                    uid:(NSString * _Nonnull)uid mic:(BOOL)enabled;
                    
/**
 *  加入通话频道
 *
 *  @param channelId       标识通话的频道名称，长度在32字节以内的字符串。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)joinChannel:(NSString * _Nonnull)channelId;

/**
 *  离开频道，即挂断或退出通话。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)leaveChannel;

/**
 *  静音/取消静音。该方法用于允许/禁止往网络发送本地音频流。
 *
 *  @param mute YES: 麦克风静音, NO: 取消静音。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)muteLocalAudioStream:(BOOL)muted;

/*开启本地音频的采集和上行
 *  @return 1: 方法调用成功，0: 方法调用失败。
*/
- (int)startLocalAudio;

/*关闭本地音频的采集和上行
 *  @return 1: 方法调用成功，0: 方法调用失败。
*/
- (int)stopLocalAudio;

/**
 *  静音指定远端用户/对指定远端用户取消静音。本方法用于允许/禁止播放远端用户的音频流。
 *
 *  @param mute YES: 停止播放指定用户的音频流, NO: 恢复播放指定用户的音频流。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)muteRemoteAudioStream:(NSString * _Nonnull)uid mute:(BOOL)muted;

/**
 *  切换音频输出方式：扬声器或听筒
 *
 *  @param enableSpeaker YES: 音频输出至扬声器，NO: 音频输出至听筒。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)setEnableSpeakerphone:(BOOL)enableSpeaker;

/**
 *  调节播放信号音量
 *
 *  @param volume 软件调节播放信号音量，默认值为100（原始音量），最大调整为200（即原始音量的2倍），如果为小于等于0，即软件级静音，无法听到任何远端音频流。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
 
- (int)adjustPlaybackSignalVolume:(NSUInteger)volume;

/**
 *  调节录音信号音量
 *
 *  @param volume 软件调节录音信号音量，默认值为100（原始音量），最大调整为400（即原始音量的4倍）；如果为小于等于0，即软件级静音，对方将听不到你说话声音；
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)adjustRecordingSignalVolume:(NSUInteger)volume;


/**
 *  启用说话者音量提示
 *
 *  @param interval 指定音量提示的时间间隔（<=0: 禁用音量提示功能；>=10: 提示间隔，单位为毫秒。建议设置到大于500毫秒。）
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)enableAudioVolumeIndication:(NSInteger)interval;

/**
 *  设置本地视频显示属性
 *
 *  @param local 本地视频显示属性
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)setupLocalVideo:(BMVideoCanvas* _Nullable)local;


/**
 *  设置远端视频显示属性
 *
 *  @param remote 远端视频显示属性
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)setupRemoteVideo:(BMVideoCanvas* _Nullable)remote;

/**
 *  启动本地视频预览
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)startPreview;

/**
 *  停止本地视频预览
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)stopPreview;

/**
 *  切换前置/后置摄像头
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)switchCamera;

/**
 *  开关摄像头闪光灯
 *
 *  @param enabled YES: 打开闪光灯，NO: 关闭闪光灯。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */

- (int)enableCameraTorchOn:(BOOL)enabled;

/**
 *  允许/禁止往网络发送本地视频流。该方法用于只看不发的视频场景。
 *
 *  @param enabled YES: 允许发送本地视频（默认），NO: 禁止发送本地视频。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
 
- (int)muteLocalVideoStream:(BOOL)muted;


/**
 *  允许/禁止播放指定的远端视频流
 *
 *  @param uid  用户ID
 *  @param mute YES: 停止播放接收到的视频流，NO: 允许播放接收到的视频流。
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)muteRemoteVideoStream:(NSString * _Nonnull)uid mute:(BOOL)muted;
- (int)muteRemoteVideoStream2:(NSString * _Nonnull)uid mute:(BOOL)muted;

/**
 *  开关视频美颜
 *
 *  @param enabled YES: 打开视频美颜，NO: 关闭视频美颜。
 *
 */
- (int)enableBeautify:(BOOL)enabled;

/**
 *  设置美颜等级
 *
 *  @param value 在0～1.0之间。
 *
 */
- (int)setBeautifyLevel:(float)value;

- (int)enableFrontOutputMirror:(BOOL)enabled;


/**
 *  设置远端显示模式
 *
 *  @param uid  远端用户id.
 *  @param mode Hidden(1), Fit(2)。 参考 MapleRtcRenderMode
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)setRemoteRenderMode:(NSString * _Nonnull)uid
                      mode:(MapleRtcRenderMode) mode;

/**
 *  设置本地显示模式
 *
 *  @param mode Hidden(1), Fit(2)。 参考 MapleRtcRenderMode
 *
 *  @return 1: 方法调用成功，0: 方法调用失败。
 */
- (int)setLocalRenderMode:(MapleRtcRenderMode) mode;

- (int)startPublishStream:(NSString * _Nonnull)name host:(NSString * _Nonnull)hostUrl hasVideo:(BOOL)videoTag;
- (int)stopPublishStream;

/**
 *  开始播放媒体文件
 *
 *  @param name  播放文件路径或者URL地址
 *  @param cycle 循环次数
 *  @param sourceType 仅广播 1， 仅本地播放2， 即广播又本地播放3
 *
*/
- (int)startPlayMedia:(NSString * _Nonnull)name type:(int)sourceType cycle:(int)cycle view:(BMVideoCanvas* _Nullable)local index:(int)index;


/**
 *  停止播放媒体文件
 *
*/
- (int)stopPlayMedia:(int)index;

/**
 *  静音媒体文件
 *
*/
- (int)mutePlayMedia:(BOOL)muted index:(int)index;

/**
 *  暂停媒体文件
 *
*/
- (int)pausePlayMedia:(BOOL)paused index:(int)index;

/**
 *  调节播放媒体文件音量
 *
*/
- (int)adjustPlayMediaVolume:(int)volume index:(int)index;

- (int)setPlayMediaPosition:(int)pos index:(int)index;

- (int)getPlayMediaTotalDuration:(int)index;

- (int)getPlayMediaCurrentPosition:(int)index;

- (int)setAudioTrackId:(int)audioTrackIs index:(int)index;

- (int)getPlayingAudioTrackId:(int)index;

- (NSArray *)getAvailableAudioTrackIds:(int)index;


/*
  @type 1 摄像头源， 2 视频源
 */

- (int)switchLocalVideoStream:(MapleRtcVideoSourceType)type;
/**
 *  开始推流到cdn
 *
*/
- (int)startPublish:(NSString * _Nonnull)uploadUrl;

/**
 *  停止推流到cdn
 *
*/
- (int)stopPublish;
@end
